/*
 * This file is a part of the xnetsentry project.
 * Copyright (C) 1998 Martin Gall
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/*
 *
 */

#ifndef A_SYSDEP_H
#define A_SYSDEP_H	1

#include "a_config.h"

#ifdef WITH_MG_KLUDGE
# undef HAVE_BITFIELDS
# define HAVE_STDARG_H
#endif

#ifdef HAVE_VOID_FUNC
# define VOID_FUNC	void
#else
# define VOID_FUNC	int
#endif

#ifdef HAVE_VOID_PTR
# define VOID_PTR	void*
#else
# define VOID_PTR	char*
#endif

#ifdef HAVE_VOID_DECL
# define VOID_DECL	void
#else
# define VOID_DECL
#endif

#ifndef HAVE_BZERO
# define bzero(ptr,size)	memset(ptr,0,size)
#endif

#ifndef HAVE_BCOPY
# define bcopy(src,dst,size)	memcpy(dst,src,size)
#endif

#ifdef NO_PROTOTYPE
# define P(proto) ()
#else
# ifdef HAVE_PROTOTYPES
#  define P(proto) proto
# else
#  define P(proto) ()
# endif
#endif

#ifdef HAVE_ANSI_CPP
# define QUOTE(expr)	#expr
#else
# define QUOTE(expr)	""
#endif

#ifndef NULL
# define NULL	((VOID_PTR)0)
#endif

#ifndef TRUE
# define TRUE	(1==1)
#endif
#ifndef FALSE
# define FALSE	(1==0)
#endif

/*
 * types
 */

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>

#endif

typedef int		t_status;
typedef int		t_boolean;

typedef int		t_32;
typedef unsigned int	t_u32;
typedef short		t_16;
typedef unsigned short	t_u16;
typedef char		t_8;
typedef unsigned char	t_u8;

#define ARRAY_COUNT(array)	(sizeof (array)/sizeof ((array)[0]))
#define OFFSET(ptype,field)	((off_t)(&(((ptype)NULL)->field)))

/*
 * stdlib
 */

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#else
extern VOID_PTR	malloc P((size_t size));
extern VOID_PTR	realloc P((VOID_PTR ptr,size_t new_size));
extern int	abort P((VOID_DECL));
extern char	*getenv P((char *var));
#endif

/*
 * unistd
 */

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#else
extern char	*ttyname();
#endif

/*
 * errno
 */

#include <errno.h>

extern int	errno;
extern char	*sys_errlist[];

#ifndef HAVE_STRERROR
char		*strerror P((int errcode));
#endif

/*
 * varargs
 */

#ifdef HAVE_STDARG_H
# include <stdarg.h>
#else
# include <varargs.h>
#endif

/*
 * BUFSIZ
 */

#include <stdio.h>

/*
 * string*.h
 */

#ifdef HAVE_STRING_H
# include <string.h>
#endif
#ifdef HAVE_STRINGS_H
# include <strings.h>
#endif

#ifndef HAVE_STRDUP
char	*strdup P((char *string));
#endif

/*
 * MAXHOSTNAMELEN
 */

#include <sys/param.h>

/*
 * assert
 */

#ifdef HAVE_ASSERT_H
# include <assert.h>
#endif

/*
 * debug
 */

#ifdef DEBUG
VOID_PTR		malloc_comment();
VOID_FUNC		free_comment();

# define MALLOC(size,_comm)	malloc_comment((size),\
					       __FILE__,\
					       __LINE__,\
					       (_comm))
# define FREE(ptr,_comm)	free_comment((ptr),\
					     (_comm))
#else
# define MALLOC(size,_comm)	malloc(size)
# define FREE(ptr,_comm)	free(ptr)
#endif
     
#endif
